/**************************************************************************//**
*   @file   main.c
*   @brief  Implementation of the program's main function.
*   @author ACozma (andrei.cozma@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 706
******************************************************************************/
/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include "ADI_Driver/adi.h"
#if (USE_uCProbe == 1)
    #include "probe_com_cfg.h"
    #include "probe_com.h"
    #include "probe_rs232.h"
    #include "probe_rs232c.h"
#endif
#include <altera_avalon_pio_regs.h>
#include <altera_avalon_spi.h>     
#include <unistd.h>                
#include <system.h>                
#include <stdio.h>
#include "alt_types.h"
#include "altera_avalon_jtag_uart_regs.h"
#include "altera_avalon_jtag_uart.h"
#include "ADI_Driver/CN0202.h"
/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
//Array of pointers to device initialization functions
static pfnBoardInit BoardInitFunctions[] = { CN0202_Init };
//Array of pointers to device parameters update functions
static pfnBoardUpdateParameters BoardUpdateParametersFunctions[] = { CN0202_UpdateParameters };
//Supported boards count
#define SUPPORTED_BOARDS_NO (sizeof(BoardInitFunctions)/sizeof(pfnBoardInit))
//A switch to enable the AD board usage
static unsigned char AD_Select = 0;
/**************************************************************************//**
 * @brief Main entry point.
 *
 * @param none
******************************************************************************/
int main(void)
{
/**** Main Function Local Variables ******************************************/
	int delayCounter 		= 0;	//counter used to divide the main loop frequency
    int selectedBoard 		= 0;	//index of the currently selected AD evaluation board
    int prevSelectedBoard	= 0;	//index of the previously selected AD evaluation board
/**** Initialization Section *************************************************/
    //Clear all LEDs
	ClearLed(LED_ALL);
	//Enable uCProbe communication
#if (USE_uCProbe == 1)
	ProbeCom_Init();
	ProbeRS232_Init(0);
	ProbeRS232_RxIntEn();
#endif
/**** Program Main Loop ******************************************************/
	while (1)
	{
	    //Local variables
		int idx 	 = 0;
	    int retCode = 0;
		//Toggle LED 1 to signal that the program is working
		delayCounter++;
		if((delayCounter % 1000) == 0)
		{
			ToggleLed(LED_1);
		}
		//Determine the selected board
		selectedBoard = 0;
		for(idx = 0; idx < SUPPORTED_BOARDS_NO && AD_Select; idx++)
		{
			if(BoardInitFunctions[idx])
			{
				selectedBoard = idx + 1;
				break;
			}
		}
		//When the board selection changes initialize the new selected board
		if(selectedBoard && (prevSelectedBoard == 0))
		{
			//clear the activity and error LEDs
			ClearLed(LED_2 | LED_8);
			//initialize the device
			retCode = BoardInitFunctions[selectedBoard - 1]();
			//if the initialization fails turn on LED 8 and clear the board selection
			if(retCode == FAIL)
			{
				SetLed(LED_8);
				AD_Select = 0;
				selectedBoard = 0;
			}
		}
		//Update the parameters of the currently selected board
		if(selectedBoard)
		{
			//blink LED 2 to indicate activity
			ToggleLed(LED_2);
			//update the board parameters
			retCode = BoardUpdateParametersFunctions[selectedBoard - 1]();
			//if the parameters update fails turn on LED 8 and clear the board selection
			if(retCode == FAIL)
			{
				SetLed(LED_8);
				ClearLed(LED_2);
				AD_Select = 0;
				selectedBoard = 0;
			}
		}
		else if(prevSelectedBoard)
		{
			//turn off LED 2 to indicate that no board is currently selected
			ClearLed(LED_2);
		}
		//Update the previous board selection
		prevSelectedBoard = selectedBoard;
	}
	return 0;
}
